from PyQt6.QtGui import QAction
import sys
import pathlib

from PyQt6.QtCore import QSize, Qt
from PyQt6.QtWidgets import (
    QApplication,
    QGridLayout,
    QPushButton,
    QToolBar,
    QWidget,
    QMainWindow,
)
import tomli
import uldaq
from IPython import embed
import numpy as np

from pyrelacs.util.logging import config_logging

log = config_logging()


class PyRelacs(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("PyRelacs")

        self.daq_connect_button = QPushButton("Connect Daq")
        self.daq_connect_button.setCheckable(True)
        self.daq_connect_button.clicked.connect(self.connect_dac)

        self.daq_disconnect_button = QPushButton("Disconnect Daq")
        self.daq_disconnect_button.setCheckable(True)
        self.daq_disconnect_button.clicked.connect(self.disconnect_dac)

        self.repro_button = QPushButton("Load Repros")
        self.repro_button.setCheckable(True)
        self.repro_button.clicked.connect(self.repro)

        layout = QGridLayout()
        layout.addWidget(self.daq_connect_button, 0, 0)
        layout.addWidget(self.daq_disconnect_button, 0, 1)
        layout.addWidget(self.repro_button, 1, 0)

        self.toolbar = QToolBar("Repros")
        self.addToolBar(self.toolbar)

        self.setFixedSize(QSize(400, 300))
        widget = QWidget()
        widget.setLayout(layout)
        self.setCentralWidget(widget)

    def connect_dac(self):
        devices = uldaq.get_daq_device_inventory(uldaq.InterfaceType.USB)
        try:
            self.daq_device = uldaq.DaqDevice(devices[0])
            log.debug(f"Found daq devices {len(devices)}, connecting to the first one")
            self.daq_device.connect()
            log.debug("Connected")
        except IndexError:
            log.debug("DAQ is not connected, closing")
            QApplication.quit()
        self.daq_connect_button.setDisabled(True)

    def disconnect_dac(self):
        try:
            log.debug(f"{self.daq_device}")
            self.daq_device.disconnect()
            self.daq_device.release()
            log.debug(f"{self.daq_device}")
            self.daq_disconnect_button.setDisabled(True)
            self.daq_connect_button.setEnabled(True)
        except AttributeError:
            log.debug("DAQ was not connected")

    def repro(self):
        config_path = pathlib.Path(__file__).parent.resolve() / "repro.toml"
        if not config_path.is_file:
            log.error("repro.toml not found")
            QApplication.quit()
        repors = []

        with open(config_path, mode="rb") as fp:
            self.config = tomli.load(fp)
            for r in self.config:
                repors.append(r)

        individual_repro_button = QAction(f"{repors[0]}", self)
        individual_repro_button.setStatusTip("Button")
        individual_repro_button.triggered.connect(self.run_repro)
        self.toolbar.addAction(individual_repro_button)

    def run_repro(self):
        log.debug("running repro")
        time = np.arange(
            0, self.config["Sinus"]["duration"], 1 / self.config["Sinus"]["fs"]
        )
        data = self.config["Sinus"]["amplitude"] * np.sin(
            2 * np.pi * self.config["Sinus"]["freq"] * time
        )

        ao_device = self.daq_device.get_ao_device()
        ao_info = ao_device.get_info()
        try:
            err = ao_device.a_out_list(
                0, 0, [uldaq.Range.BIP2VOLTS], uldaq.AOutListFlag.DEFAULT, data
            )
        except uldaq.ULException as e:
            print(e)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = PyRelacs()
    window.show()
    app.exec()
